library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;

use work.gencores_pkg.all;
use work.vme64x_pkg.all;

library unisim;
use unisim.vcomponents.all;

entity svec7_sfpga_bridge is
  generic(
    g_clock_period : integer
    );
  port (

    afpga_rst_n_i : in std_logic;
    afpga_clk_i   : in std_logic;

    afpga_d_i     : in  std_logic_vector(7 downto 0);
    afpga_frame_i : in  std_logic;
    afpga_d_o     : out std_logic_vector(7 downto 0);
    afpga_frame_o : out std_logic;

    vme_i : in  t_vme64x_in;
    vme_o : out t_vme64x_out
    );

end svec7_sfpga_bridge;

architecture rtl of svec7_sfpga_bridge is

  
  constant c_tag_addr     : std_logic_vector(7 downto 0) := x"a0";
  constant c_tag_read     : std_logic_vector(7 downto 0) := x"a1";
  constant c_tag_write    : std_logic_vector(7 downto 0) := x"a2";
  constant c_tag_read_cpl : std_logic_vector(7 downto 0) := x"a3";
  constant c_tag_irq      : std_logic_vector(7 downto 0) := x"a4";
  constant c_tag_iackin   : std_logic_vector(7 downto 0) := x"a5";
  constant c_tag_iackout  : std_logic_vector(7 downto 0) := x"a6";
  constant c_tag_aux     : std_logic_vector(7 downto 0) := x"a7";
  constant c_tag_release : std_logic_vector(7 downto 0) := x"a8";
  
  signal as_fedge_p : std_logic;
  signal as_redge_p : std_logic;
  signal ds_fedge_p : std_logic_vector(1 downto 0);
  signal ds_synced  : std_logic_vector(1 downto 0);

  signal ADDR_latched    : std_logic_vector(31 downto 1);
  signal LWORD_latched_n : std_logic;
  signal AM_latched      : std_logic_vector(5 downto 0);
  signal WRITE_latched   : std_logic;
  signal data_latched    : std_logic_vector(63 downto 0);
  signal ds_latched      : std_logic_vector(1 downto 0);

  signal release_req : std_logic;
  signal release_ack : std_logic;
  signal addr_req, addr_ack   : std_logic;
  signal write_req, write_ack : std_logic;
  signal read_req, read_ack   : std_logic;
  signal rdata_req, rdata_ack : std_logic;

  signal rx_dtack, rx_berr : std_logic;
  signal as_n_synced : std_logic;
  
  signal ds_count : unsigned(2 downto 0);

  -- Calculate the number of LATCH DS states necessary to match the timing
  -- rule 2.39 page 113 VMEbus specification ANSI/IEEE STD1014-1987.
  -- (max skew for the slave is 20 ns)
  constant c_num_latchDS : natural range 1 to 8 :=
    (20 + g_CLOCK_PERIOD - 1) / g_CLOCK_PERIOD;

  signal afpga_dout      : std_logic_vector(15 downto 0);
  signal afpga_din      : std_logic_vector(15 downto 0);
  signal afpga_frame_out : std_logic;
  signal afpga_frame_in : std_logic_vector(1 downto 0);
  signal afpga_clk_n     : std_logic;

  type t_ds_fsm_state is
    (
      IDLE,
      REQ_READ,
      REQ_WRITE,
      REQ_WRITE_ACK);

  type t_bridge_tx_state is
    (IDLE,
     TX_ADDR0,
     TX_ADDR1,
     TX_WRITE0,
     TX_WRITE1,
     TX_WRITE2,
     TX_WRITE3);

  type t_bridge_rx_state is
    (
      IDLE,
      RX_DATA0,
      RX_DATA1,
      RX_DATA2,
      RX_DATA3,
      GEN_DTACK);


  signal ds_state : t_ds_fsm_state;
  signal tx_state : t_bridge_tx_state;
  signal rx_state : t_bridge_rx_state;

begin

  afpga_clk_n <= not afpga_clk_i;

  gen_ddr_ios : for i in 0 to 7 generate

    U_ODDR_Data : ODDR2
      generic map (
        DDR_ALIGNMENT => "C0")
      port map (
        Q  => afpga_d_o(i),
        C0 => afpga_clk_i,
        C1 => afpga_clk_n,
        CE => '1',
        D0 => afpga_dout(2*i),
        D1 => afpga_dout(2*i+1),
        R  => '0',
        S  => '0');

    U_IDDR_Data : IDDR2
      generic map (
        DDR_ALIGNMENT => "C0" )
      port map (
        Q0 => afpga_din(2*i),
        Q1 => afpga_din(2*i+1),
        C0 => afpga_clk_i,
        C1 => afpga_clk_n,
        CE => '1',
        D  => afpga_d_i(i),
        R  => '0',
        S  => '0');
    
  end generate gen_ddr_ios;

  U_IDDR_Frame : IDDR2
      generic map (
        DDR_ALIGNMENT => "C0" )
      port map (
        Q0 => afpga_frame_in(0),
        Q1 => afpga_frame_in(1),
        C0 => afpga_clk_i,
        C1 => afpga_clk_n,
        CE => '1',
        D  => afpga_frame_i,
        R  => '0',
        S  => '0');
        
  U_ODDR_Frame : ODDR2
    generic map (
      DDR_ALIGNMENT => "C0")
    port map (
      Q  => afpga_frame_o,
      C0 => afpga_clk_i,
      C1 => afpga_clk_n,
      CE => '1',
      D0 => afpga_frame_out,
      D1 => afpga_frame_out,
      R  => '0',
      S  => '0');

  U_Sync1 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.as_n,
      synced_o => as_n_synced,
      npulse_o => as_fedge_p);

  U_Sync2 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.ds_n(0),
      synced_o => ds_synced(0),
      npulse_o => ds_fedge_p(0));

  U_Sync3 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.ds_n(1),
      synced_o => ds_synced(1),
      npulse_o => ds_fedge_p(1));

  p_latch_addr : process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if afpga_rst_n_i = '0' then
        addr_req <= '0';
        release_req <= '0';
      elsif as_redge_p = '1' then
        release_req <= '1';
      elsif as_fedge_p = '1' then
        ADDR_latched    <= vme_i.Addr;
        LWORD_latched_n <= vme_i.LWORD_n;
        AM_latched      <= vme_i.am;
        addr_req        <= '1';
      elsif addr_ack = '1' then
        addr_req <= '0';
      end if;
    end if;
  end process;

  p_latch_data : process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if afpga_rst_n_i = '0' then
        read_req  <= '0';
        write_req <= '0';
        ds_state  <= IDLE;
      else
        case ds_state is
          when IDLE =>
            read_req  <= '0';
            write_req <= '0';
            if ds_fedge_p /= "00" and as_n_synced = '0' then
              if vme_i.write_n = '0' then
                ds_state <= REQ_WRITE;
                ds_count <= (others => '0');
              else
                ds_state <= REQ_READ;
                read_req <= '1';
              end if;
            end if;

          when REQ_READ =>
            if read_ack = '1' or ds_synced = "11" then
              ds_state <= IDLE;
              read_req <= '0';
            end if;

          when REQ_WRITE =>
            ds_count <= ds_count + 1;
            if ds_synced = "11" then
              ds_state <= IDLE;
            elsif ds_count = c_num_latchDS then
              write_req                  <= '1';
              ds_latched                 <= vme_i.ds_n;
              data_latched(63 downto 33) <= vme_i.addr;
              data_latched(32)           <= vme_i.LWORD_n;
              data_latched(31 downto 0)  <= vme_i.data;
              ds_state                   <= REQ_WRITE_ACK;
            end if;

          when REQ_WRITE_ACK =>
            if write_ack = '1' then
              write_req <= '0';
              ds_state  <= IDLE;
            end if;

        end case;
      end if;
    end if;
  end process;


  p_in_fsm : process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if afpga_rst_n_i = '0' then
        rx_state <= IDLE;

        case rx_state is
          when IDLE =>
            if afpga_frame_i = '1' then

              case afpga_din(15 downto 8) is
                when c_tag_read_cpl =>
                  rx_dtack <= afpga_din(0);
                  rx_berr  <= afpga_din(1);
                  rx_state <= RX_DATA0;

                when c_tag_iackin => null;
                when others       => null;
              end case;


            end if;
            when others => null;
        end case;
      end if;
    end if;
  end process;


  p_out_fsm : process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if afpga_rst_n_i = '0' then
        tx_state        <= IDLE;
        afpga_dout      <= (others => '0');
        afpga_frame_out <= '0';
        addr_ack        <= '0';
      else
        case tx_state is
          when IDLE =>
            write_ack <= '0';
            read_ack  <= '0';
            addr_ack  <= '0';
            afpga_frame_out <= '0';
            afpga_dout <= (others => '0');
            if addr_req = '1' then
              afpga_dout(15 downto 8) <= c_tag_addr;
              afpga_dout(7 downto 2)  <= AM_latched;
              afpga_dout(1) <= '0';
              afpga_dout(0)           <= LWORD_latched_n;
              afpga_frame_out         <= '1';
              tx_state                <= TX_ADDR0;
            elsif read_req = '1' and read_ack = '0' then
              afpga_dout(15 downto 8) <= c_tag_read;
              afpga_dout(1 downto 0)  <= ds_latched;
              afpga_frame_out         <= '1';
              read_ack                <= '1';
              tx_state                <= IDLE;
            elsif write_req = '1' and write_ack = '0' then
              afpga_dout(15 downto 8) <= c_tag_write;
              afpga_dout(1 downto 0)  <= ds_latched;
              afpga_frame_out         <= '1';
              tx_state                <= TX_WRITE0;
            else
              afpga_frame_out <= '1';
              afpga_dout(15 downto 8) <= c_tag_aux;
              afpga_dout(5 downto 0) <= vme_i.ga;
              afpga_dout(6) <= vme_i.iack_n;
              afpga_dout(7) <= vme_i.iackin_n;
              tx_state <= IDLE;
            end if;

          when TX_WRITE0 =>
            afpga_dout      <= data_latched(63 downto 48);
            afpga_frame_out <= '0';
            tx_state        <= TX_WRITE1;
          when TX_WRITE1 =>
            afpga_dout      <= data_latched(47 downto 32);
            afpga_frame_out <= '0';
            tx_state        <= TX_WRITE2;
          when TX_WRITE2 =>
            afpga_dout      <= data_latched(31 downto 16);
            afpga_frame_out <= '0';
            tx_state        <= TX_WRITE3;
          when TX_WRITE3 =>
            afpga_dout <= data_latched(15 downto 0);
            write_ack  <= '1';
            tx_state   <= IDLE;

          when TX_ADDR0 =>
            afpga_dout      <= ADDR_latched(31 downto 16);
            afpga_frame_out <= '0';
            tx_state        <= TX_ADDR1;
            addr_ack        <= '1';

          when TX_ADDR1 =>
            afpga_dout      <= ADDR_latched(15 downto 1) & '0';
            afpga_frame_out <= '0';
            addr_ack        <= '0';
            tx_state        <= IDLE;
        end case;






      end if;
    end if;
  end process;


  vme_o.dtack_n  <= '1';
  vme_o.berr_n   <= '1';
  vme_o.dtack_oe <= '1';

  vme_o.data_dir  <= '0';
  vme_o.addr_dir  <= '0';
  vme_o.addr_oe_n <= '0';
  vme_o.data_oe_n <= '0';


end rtl;
